package jrummikub.control.network;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import jrummikub.util.Connection;
import jrummikub.util.Event;
import jrummikub.util.IEvent;
import jrummikub.util.IListener;
import jrummikub.util.IListener1;
import jrummikub.util.LoginData;
import jrummikub.view.IGameListPanel;
import jrummikub.view.IGameListPanel.GameData;
import jrummikub.view.IView;

public class NetworkControl {
	private ConnectionControl connectionControl;
	private IView view;
	private List<Connection> connections = new ArrayList<Connection>();
	private Event stopNetworkEvent = new Event();

	private Map<UUID, GameData> gameMap = new HashMap<UUID, GameData>();

	public NetworkControl(final LoginData loginData, final IView view) {
		this.view = view;
		connectionControl = new ConnectionControl(loginData);

		connections.add(connectionControl.getConnectedEvent().add(new IListener() {
			@Override
			public void handle() {
				view.getGameListPanel().setChannelName(loginData.getChannelName());
				view.showGameListPanel(true);

				GameData testData = new GameData(UUID.randomUUID(), "NeoRaider");
				testData.setCurrentPlayerCount(2);
				testData.setMaxPlayerCount(4);

				connectionControl.offerGame(testData);
			}
		}));

		connections.add(connectionControl.getConnectionFailedEvent().add(
				new IListener() {
					@Override
					public void handle() {
						// TODO Auto-generated method stub

					}
				}));

		connections.add(connectionControl.getGameOfferEvent().add(
				new IListener1<IGameListPanel.GameData>() {
					@Override
					public void handle(GameData value) {
						GameData game = gameMap.get(value.getGameID());

						if (game == null) {
							game = value;
							gameMap.put(value.getGameID(), value);
						} else {
							game.setCurrentPlayerCount(value.getCurrentPlayerCount());
							game.setMaxPlayerCount(value.getMaxPlayerCount());
						}

						view.getGameListPanel().addGame(game);
					}
				}));
		connections.add(connectionControl.getGameWithdrawalEvent().add(
				new IListener1<UUID>() {
					@Override
					public void handle(UUID value) {
						GameData game = gameMap.get(value);

						if (game != null) {
							view.getGameListPanel().removeGame(game);
							gameMap.remove(value);
						}
					}
				}));

		connections.add(view.getGameListPanel().getJoinEvent()
				.add(new IListener1<IGameListPanel.GameData>() {
					@Override
					public void handle(GameData value) {
						// TODO Auto-generated method stub

					}
				}));

		connections.add(view.getGameListPanel().getOpenNewGameEvent()
				.add(new IListener() {
					@Override
					public void handle() {
						// TODO Auto-generated method stub

					}
				}));

		connections.add(view.getGameListPanel().getCancelEvent()
				.add(new IListener() {
					@Override
					public void handle() {
						abort();
						stopNetworkEvent.emit();
					}
				}));
	}

	public void startNetwork() {
		connectionControl.connect();
	}

	public void abort() {
		for (Connection c : connections) {
			c.remove();
		}
		connectionControl.disconnect();
		view.showGameListPanel(false);
	}

	public IEvent getStopNetworkEvent() {
		return stopNetworkEvent;
	}

}
